#!/usr/bin/env python3

from bleak import BleakClient

ON = [0x4, 0x4, 0x2]
OFF = [0x2, 0x0]
SLEEP = [0x4, 0x1]
HR = [0x4, 0x2]
SPD = [0x4, 0x3]
MIN_SPEED = [0x2, 0x1]
HALF_SPEED = [0x2, 0x32]
FULL_SPEED = [0x2, 0x64]
CHARACTERISTIC = "a026e038-0a7d-4ab3-97fa-f1500f9feb8b"
# > read the above characteristic 0xFD-01-XX-04 where XX is speed 
# > bytearray(b'\xfd\x01\x00\x01')

class Headwind:
    def __init__(self, address):
        self.address = address

    async def readSpeed(self):
        try:
            async with BleakClient(self.address) as client:
                result = await client.read_gatt_char(CHARACTERISTIC)
                return result[2]
        except Exception:
            return 0

    async def readMode(self):
        try:
            async with BleakClient(self.address) as client:
                result = await client.read_gatt_char(CHARACTERISTIC)
                return result[3] # Return state code, needs to figure out what each code means
        except Exception:
            return 0

    async def writeSleep(self):
        try:
            async with BleakClient(self.address) as client:
                await client.write_gatt_char(CHARACTERISTIC, SLEEP, response=False)
                return True
        except Exception as e:
            return False
    
    async def writeOn(self):
        try:
            async with BleakClient(self.address) as client:
                await client.write_gatt_char(CHARACTERISTIC, ON, response=False)
                return True
        except Exception as e:
            return False

    async def writeSpeedMode(self):
        try:
            async with BleakClient(self.address) as client:
                await client.write_gatt_char(CHARACTERISTIC, SPD, response=False)
                return True
        except Exception as e:
            return False

    async def writeHrMode(self):
        try:
            async with BleakClient(self.address) as client:
                await client.write_gatt_char(CHARACTERISTIC, HR, response=False)
                return True
        except Exception as e:
            return False

    async def writeSpeed(self, speed):
        if speed > 0:
            value = [0x2, speed]
            try:
                async with BleakClient(self.address) as client:
                    await client.write_gatt_char(CHARACTERISTIC, value, response=False)
                    return True
            except Exception as e:
                return False
        return False

    async def writeOff(self):
        try:
            async with BleakClient(self.address) as client:
                await client.write_gatt_char(CHARACTERISTIC, OFF, response=False)
                return True
        except Exception as e:
            return False